@@ -19,164 +19,81 @@ import warnings
 import hashlib
 import logging
 import shlex
+import re
+import os
+from collections import OrderedDict
+from collections.abc import MutableMapping
 from math import floor
 
 from botocore.vendored import six
 from botocore.exceptions import MD5UnavailableError
+from dateutil.tz import tzlocal
 from urllib3 import exceptions
 
 logger = logging.getLogger(__name__)
 
 
-if six.PY3:
-    from botocore.vendored.six.moves import http_client
+from botocore.vendored.six.moves import http_client
 
-    class HTTPHeaders(http_client.HTTPMessage):
-        pass
+class HTTPHeaders(http_client.HTTPMessage):
+    pass
 
-    from urllib.parse import quote
-    from urllib.parse import urlencode
-    from urllib.parse import unquote
-    from urllib.parse import unquote_plus
-    from urllib.parse import urlparse
-    from urllib.parse import urlsplit
-    from urllib.parse import urlunsplit
-    from urllib.parse import urljoin
-    from urllib.parse import parse_qsl
-    from urllib.parse import parse_qs
-    from http.client import HTTPResponse
-    from io import IOBase as _IOBase
-    from base64 import encodebytes
-    from email.utils import formatdate
-    from itertools import zip_longest
-    file_type = _IOBase
-    zip = zip
+from urllib.parse import (
+    quote,
+    urlencode,
+    unquote,
+    unquote_plus,
+    urlparse,
+    urlsplit,
+    urlunsplit,
+    urljoin,
+    parse_qsl,
+    parse_qs,
+)
+from http.client import HTTPResponse
+from io import IOBase as _IOBase
+from base64 import encodebytes
+from email.utils import formatdate
+from itertools import zip_longest
+file_type = _IOBase
+zip = zip
 
-    # In python3, unquote takes a str() object, url decodes it,
-    # then takes the bytestring and decodes it to utf-8.
-    # Python2 we'll have to do this ourself (see below).
-    unquote_str = unquote_plus
+# In python3, unquote takes a str() object, url decodes it,
+# then takes the bytestring and decodes it to utf-8.
+unquote_str = unquote_plus
 
-    def set_socket_timeout(http_response, timeout):
-        """Set the timeout of the socket from an HTTPResponse.
+def set_socket_timeout(http_response, timeout):
+    """Set the timeout of the socket from an HTTPResponse.
 
-        :param http_response: An instance of ``httplib.HTTPResponse``
+    :param http_response: An instance of ``httplib.HTTPResponse``
 
-        """
-        http_response._fp.fp.raw._sock.settimeout(timeout)
+    """
+    http_response._fp.fp.raw._sock.settimeout(timeout)
 
-    def accepts_kwargs(func):
-        # In python3.4.1, there's backwards incompatible
-        # changes when using getargspec with functools.partials.
-        return inspect.getfullargspec(func)[2]
+def accepts_kwargs(func):
+    # In python3.4.1, there's backwards incompatible
+    # changes when using getargspec with functools.partials.
+    return inspect.getfullargspec(func)[2]
 
-    def ensure_unicode(s, encoding=None, errors=None):
-        # NOOP in Python 3, because every string is already unicode
+def ensure_unicode(s, encoding=None, errors=None):
+    # NOOP in Python 3, because every string is already unicode
+    return s
+
+def ensure_bytes(s, encoding='utf-8', errors='strict'):
+    if isinstance(s, str):
+        return s.encode(encoding, errors)
+    if isinstance(s, bytes):
         return s
+    raise ValueError(f"Expected str or bytes, received {type(s)}.")
 
-    def ensure_bytes(s, encoding='utf-8', errors='strict'):
-        if isinstance(s, str):
-            return s.encode(encoding, errors)
-        if isinstance(s, bytes):
-            return s
-        raise ValueError("Expected str or bytes, received %s." % type(s))
-
-else:
-    from urllib import quote
-    from urllib import urlencode
-    from urllib import unquote
-    from urllib import unquote_plus
-    from urlparse import urlparse
-    from urlparse import urlsplit
-    from urlparse import urlunsplit
-    from urlparse import urljoin
-    from urlparse import parse_qsl
-    from urlparse import parse_qs
-    from email.message import Message
-    from email.Utils import formatdate
-    file_type = file
-    from itertools import izip as zip
-    from itertools import izip_longest as zip_longest
-    from httplib import HTTPResponse
-    from base64 import encodestring as encodebytes
-
-    class HTTPHeaders(Message):
-
-        # The __iter__ method is not available in python2.x, so we have
-        # to port the py3 version.
-        def __iter__(self):
-            for field, value in self._headers:
-                yield field
-
-    def unquote_str(value, encoding='utf-8'):
-        # In python2, unquote() gives us a string back that has the urldecoded
-        # bits, but not the unicode parts.  We need to decode this manually.
-        # unquote has special logic in which if it receives a unicode object it
-        # will decode it to latin1.  This is hard coded.  To avoid this, we'll
-        # encode the string with the passed in encoding before trying to
-        # unquote it.
-        byte_string = value.encode(encoding)
-        return unquote_plus(byte_string).decode(encoding)
-
-    def set_socket_timeout(http_response, timeout):
-        """Set the timeout of the socket from an HTTPResponse.
-
-        :param http_response: An instance of ``httplib.HTTPResponse``
-
-        """
-        http_response._fp.fp._sock.settimeout(timeout)
-
-    def accepts_kwargs(func):
-        return inspect.getargspec(func)[2]
-
-    def ensure_unicode(s, encoding='utf-8', errors='strict'):
-        if isinstance(s, six.text_type):
-            return s
-        return unicode(s, encoding, errors)
-
-    def ensure_bytes(s, encoding='utf-8', errors='strict'):
-        if isinstance(s, unicode):
-            return s.encode(encoding, errors)
-        if isinstance(s, str):
-            return s
-        raise ValueError("Expected str or unicode, received %s." % type(s))
 
 try:
-    from collections import OrderedDict
+    import xml.etree.cElementTree as ETree
 except ImportError:
-    # Python2.6 we use the 3rd party back port.
-    from ordereddict import OrderedDict
-
-
-if sys.version_info[:2] == (2, 6):
-    import simplejson as json
-    # In py26, invalid xml parsed by element tree
-    # will raise a plain old SyntaxError instead of
-    # a real exception, so we need to abstract this change.
-    XMLParseError = SyntaxError
-
-    # Handle https://github.com/shazow/urllib3/issues/497 for py2.6.  In
-    # python2.6, there is a known issue where sometimes we cannot read the SAN
-    # from an SSL cert (http://bugs.python.org/issue13034).  However, newer
-    # versions of urllib3 will warn you when there is no SAN.  While we could
-    # just turn off this warning in urllib3 altogether, we _do_ want warnings
-    # when they're legitimate warnings.  This method tries to scope the warning
-    # filter to be as specific as possible.
-    def filter_ssl_san_warnings():
-        warnings.filterwarnings(
-            'ignore',
-            message="Certificate has no.*subjectAltName.*",
-            category=exceptions.SecurityWarning,
-            module=r".*urllib3\.connection")
-else:
-    import xml.etree.cElementTree
-    XMLParseError = xml.etree.cElementTree.ParseError
-    import json
-
-    def filter_ssl_san_warnings():
-        # Noop for non-py26 versions.  We will parse the SAN
-        # appropriately.
-        pass
+    # cElementTree does not exist from Python3.9+
+    import xml.etree.ElementTree as ETree
+XMLParseError = ETree.ParseError
+import json
 
 
 def filter_ssl_warnings():
@@ -185,8 +102,8 @@ def filter_ssl_warnings():
         'ignore',
         message="A true SSLContext object is not available.*",
         category=exceptions.InsecurePlatformWarning,
-        module=r".*urllib3\.util\.ssl_")
-    filter_ssl_san_warnings()
+        module=r".*urllib3\.util\.ssl_",
+    )
 
 
 @classmethod
@@ -204,25 +121,16 @@ def from_pairs(cls, pairs):
         new_instance[key] = value
     return new_instance
 
+
 HTTPHeaders.from_dict = from_dict
 HTTPHeaders.from_pairs = from_pairs
 
 
 def copy_kwargs(kwargs):
     """
-    There is a bug in Python versions < 2.6.5 that prevents you
-    from passing unicode keyword args (#4978).  This function
-    takes a dictionary of kwargs and returns a copy.  If you are
-    using Python < 2.6.5, it also encodes the keys to avoid this bug.
-    Oh, and version_info wasn't a namedtuple back then, either!
+    This used to be a compat shim for 2.6 but is now just an alias.
     """
-    vi = sys.version_info
-    if vi[0] == 2 and vi[1] <= 6 and vi[3] < 5:
-        copy_kwargs = {}
-        for key in kwargs:
-            copy_kwargs[key.encode('utf-8')] = kwargs[key]
-    else:
-        copy_kwargs = copy.copy(kwargs)
+    copy_kwargs = copy.copy(kwargs)
     return copy_kwargs
 
 
@@ -230,22 +138,12 @@ def total_seconds(delta):
     """
     Returns the total seconds in a ``datetime.timedelta``.
 
-    Python 2.6 does not have ``timedelta.total_seconds()``, so we have
-    to calculate this ourselves. On 2.7 or better, we'll take advantage of the
-    built-in method.
-
-    The math was pulled from the ``datetime`` docs
-    (http://docs.python.org/2.7/library/datetime.html#datetime.timedelta.total_seconds).
+    This used to be a compat shim for 2.6 but is now just an alias.
 
     :param delta: The timedelta object
     :type delta: ``datetime.timedelta``
     """
-    if sys.version_info[:2] != (2, 6):
-        return delta.total_seconds()
-
-    day_in_seconds = delta.days * 24 * 3600.0
-    micro_in_seconds = delta.microseconds / 10.0**6
-    return day_in_seconds + delta.seconds + micro_in_seconds
+    return delta.total_seconds()
 
 
 # Checks to see if md5 is available on this system. A given system might not
@@ -365,7 +263,7 @@ def _windows_shell_split(s):
 
     # Quotes must be terminated.
     if is_quoted:
-        raise ValueError('No closing quotation in string: %s' % s)
+        raise ValueError(f"No closing quotation in string: {s}")
 
     # There may be some leftover backslashes, so we need to add them in.
     # There's no quote so we add the exact number.
@@ -379,7 +277,67 @@ def _windows_shell_split(s):
     return components
 
 
+def get_tzinfo_options():
+    # Due to dateutil/dateutil#197, Windows may fail to parse times in the past
+    # with the system clock. We can alternatively fallback to tzwininfo when
+    # this happens, which will get time info from the Windows registry.
+    if sys.platform == 'win32':
+        from dateutil.tz import tzwinlocal
+
+        return (tzlocal, tzwinlocal)
+    else:
+        return (tzlocal,)
+
+
+# Detect if CRT is available for use
 try:
-    from collections.abc import MutableMapping
+    import awscrt.auth
+
+    # Allow user opt-out if needed
+    disabled = os.environ.get('BOTO_DISABLE_CRT', "false")
+    HAS_CRT = not disabled.lower() == 'true'
 except ImportError:
-    from collections import MutableMapping
+    HAS_CRT = False
+
+
+########################################################
+#              urllib3 compat backports                #
+########################################################
+
+# Vendoring IPv6 validation regex patterns from urllib3
+# https://github.com/urllib3/urllib3/blob/7e856c0/src/urllib3/util/url.py
+IPV4_PAT = r"(?:[0-9]{1,3}\.){3}[0-9]{1,3}"
+HEX_PAT = "[0-9A-Fa-f]{1,4}"
+LS32_PAT = "(?:{hex}:{hex}|{ipv4})".format(hex=HEX_PAT, ipv4=IPV4_PAT)
+_subs = {"hex": HEX_PAT, "ls32": LS32_PAT}
+_variations = [
+    #                            6( h16 ":" ) ls32
+    "(?:%(hex)s:){6}%(ls32)s",
+    #                       "::" 5( h16 ":" ) ls32
+    "::(?:%(hex)s:){5}%(ls32)s",
+    # [               h16 ] "::" 4( h16 ":" ) ls32
+    "(?:%(hex)s)?::(?:%(hex)s:){4}%(ls32)s",
+    # [ *1( h16 ":" ) h16 ] "::" 3( h16 ":" ) ls32
+    "(?:(?:%(hex)s:)?%(hex)s)?::(?:%(hex)s:){3}%(ls32)s",
+    # [ *2( h16 ":" ) h16 ] "::" 2( h16 ":" ) ls32
+    "(?:(?:%(hex)s:){0,2}%(hex)s)?::(?:%(hex)s:){2}%(ls32)s",
+    # [ *3( h16 ":" ) h16 ] "::"    h16 ":"   ls32
+    "(?:(?:%(hex)s:){0,3}%(hex)s)?::%(hex)s:%(ls32)s",
+    # [ *4( h16 ":" ) h16 ] "::"              ls32
+    "(?:(?:%(hex)s:){0,4}%(hex)s)?::%(ls32)s",
+    # [ *5( h16 ":" ) h16 ] "::"              h16
+    "(?:(?:%(hex)s:){0,5}%(hex)s)?::%(hex)s",
+    # [ *6( h16 ":" ) h16 ] "::"
+    "(?:(?:%(hex)s:){0,6}%(hex)s)?::",
+]
+
+UNRESERVED_PAT = (
+    r"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789._!\-~"
+)
+IPV6_PAT = "(?:" + "|".join([x % _subs for x in _variations]) + ")"
+ZONE_ID_PAT = "(?:%25|%)(?:[" + UNRESERVED_PAT + "]|%[a-fA-F0-9]{2})+"
+IPV6_ADDRZ_PAT = r"\[" + IPV6_PAT + r"(?:" + ZONE_ID_PAT + r")?\]"
+IPV6_ADDRZ_RE = re.compile("^" + IPV6_ADDRZ_PAT + "$")
+
+# These are the characters that are stripped by post-bpo-43882 urlparse().
+UNSAFE_URL_CHARS = frozenset('\t\r\n')
