@@ -12,22 +12,26 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 
-import os
+import datetime
 import logging
-import time
+import os
 import threading
+import time
+import uuid
 
-from botocore.vendored import six
-
+from botocore import parsers
 from botocore.awsrequest import create_request_object
 from botocore.exceptions import HTTPClientError
-from botocore.httpsession import URLLib3Session
-from botocore.utils import is_valid_endpoint_url, get_environ_proxies
-from botocore.hooks import first_non_none_response
 from botocore.history import get_global_history_recorder
+from botocore.hooks import first_non_none_response
+from botocore.httpchecksum import handle_checksum_body
+from botocore.httpsession import URLLib3Session
 from botocore.response import StreamingBody
-from botocore import parsers
-
+from botocore.utils import (
+    get_environ_proxies,
+    is_valid_endpoint_url,
+    is_valid_ipv6_endpoint_url,
+)
 
 logger = logging.getLogger(__name__)
 history_recorder = get_global_history_recorder()
@@ -56,7 +60,7 @@ def convert_to_response_dict(http_response, operation_model):
         'status_code': http_response.status_code,
         'context': {
             'operation_name': operation_model.name,
-        }
+        },
     }
     if response_dict['status_code'] >= 300:
         response_dict['body'] = http_response.content
@@ -70,7 +74,7 @@ def convert_to_response_dict(http_response, operation_model):
     return response_dict
 
 
-class Endpoint(object):
+class Endpoint:
     """
     Represents an endpoint for a particular service in a specific
     region.  Only an endpoint can make requests.
@@ -80,8 +84,15 @@ class Endpoint(object):
     :ivar host: The fully qualified endpoint hostname.
     :ivar session: The session object.
     """
-    def __init__(self, host, endpoint_prefix, event_emitter,
-                 response_parser_factory=None, http_session=None):
+
+    def __init__(
+        self,
+        host,
+        endpoint_prefix,
+        event_emitter,
+        response_parser_factory=None,
+        http_session=None,
+    ):
         self._endpoint_prefix = endpoint_prefix
         self._event_emitter = event_emitter
         self.host = host
@@ -94,63 +105,125 @@ class Endpoint(object):
             self.http_session = URLLib3Session()
 
     def __repr__(self):
-        return '%s(%s)' % (self._endpoint_prefix, self.host)
+        return f'{self._endpoint_prefix}({self.host})'
 
     def make_request(self, operation_model, request_dict):
-        logger.debug("Making request for %s with params: %s",
-                     operation_model, request_dict)
+        logger.debug(
+            "Making request for %s with params: %s",
+            operation_model,
+            request_dict,
+        )
         return self._send_request(request_dict, operation_model)
 
     def create_request(self, params, operation_model=None):
         request = create_request_object(params)
         if operation_model:
-            request.stream_output = any([
-                operation_model.has_streaming_output,
-                operation_model.has_event_stream_output
-            ])
+            request.stream_output = any(
+                [
+                    operation_model.has_streaming_output,
+                    operation_model.has_event_stream_output,
+                ]
+            )
             service_id = operation_model.service_model.service_id.hyphenize()
             event_name = 'request-created.{service_id}.{op_name}'.format(
-                service_id=service_id,
-                op_name=operation_model.name)
-            self._event_emitter.emit(event_name, request=request,
-                                     operation_name=operation_model.name)
+                service_id=service_id, op_name=operation_model.name
+            )
+            self._event_emitter.emit(
+                event_name,
+                request=request,
+                operation_name=operation_model.name,
+            )
         prepared_request = self.prepare_request(request)
         return prepared_request
 
     def _encode_headers(self, headers):
         # In place encoding of headers to utf-8 if they are unicode.
         for key, value in headers.items():
-            if isinstance(value, six.text_type):
+            if isinstance(value, str):
                 headers[key] = value.encode('utf-8')
 
     def prepare_request(self, request):
         self._encode_headers(request.headers)
         return request.prepare()
 
+    def _calculate_ttl(
+        self, response_received_timestamp, date_header, read_timeout
+    ):
+        local_timestamp = datetime.datetime.utcnow()
+        date_conversion = datetime.datetime.strptime(
+            date_header, "%a, %d %b %Y %H:%M:%S %Z"
+        )
+        estimated_skew = date_conversion - response_received_timestamp
+        ttl = (
+            local_timestamp
+            + datetime.timedelta(seconds=read_timeout)
+            + estimated_skew
+        )
+        return ttl.strftime('%Y%m%dT%H%M%SZ')
+
+    def _set_ttl(self, retries_context, read_timeout, success_response):
+        response_date_header = success_response[0].headers.get('Date')
+        has_streaming_input = retries_context.get('has_streaming_input')
+        if response_date_header and not has_streaming_input:
+            try:
+                response_received_timestamp = datetime.datetime.utcnow()
+                retries_context['ttl'] = self._calculate_ttl(
+                    response_received_timestamp,
+                    response_date_header,
+                    read_timeout,
+                )
+            except Exception:
+                logger.debug(
+                    "Exception received when updating retries context with TTL",
+                    exc_info=True,
+                )
+
+    def _update_retries_context(self, context, attempt, success_response=None):
+        retries_context = context.setdefault('retries', {})
+        retries_context['attempt'] = attempt
+        if 'invocation-id' not in retries_context:
+            retries_context['invocation-id'] = str(uuid.uuid4())
+
+        if success_response:
+            read_timeout = context['client_config'].read_timeout
+            self._set_ttl(retries_context, read_timeout, success_response)
+
     def _send_request(self, request_dict, operation_model):
         attempts = 1
-        request = self.create_request(request_dict, operation_model)
         context = request_dict['context']
+        self._update_retries_context(context, attempts)
+        request = self.create_request(request_dict, operation_model)
         success_response, exception = self._get_response(
-            request, operation_model, context)
-        while self._needs_retry(attempts, operation_model, request_dict,
-                                success_response, exception):
+            request, operation_model, context
+        )
+        while self._needs_retry(
+            attempts,
+            operation_model,
+            request_dict,
+            success_response,
+            exception,
+        ):
             attempts += 1
+            self._update_retries_context(context, attempts, success_response)
             # If there is a stream associated with the request, we need
             # to reset it before attempting to send the request again.
             # This will ensure that we resend the entire contents of the
             # body.
             request.reset_stream()
             # Create a new request when retried (including a new signature).
-            request = self.create_request(
-                request_dict, operation_model)
+            request = self.create_request(request_dict, operation_model)
             success_response, exception = self._get_response(
-                request, operation_model, context)
-        if success_response is not None and \
-                'ResponseMetadata' in success_response[1]:
+                request, operation_model, context
+            )
+        if (
+            success_response is not None
+            and 'ResponseMetadata' in success_response[1]
+        ):
             # We want to share num retries, not num attempts.
             total_retries = attempts - 1
-            success_response[1]['ResponseMetadata']['RetryAttempts'] = total_retries
+            success_response[1]['ResponseMetadata'][
+                'RetryAttempts'
+            ] = total_retries
         if exception is not None:
             raise exception
         else:
@@ -163,7 +236,8 @@ class Endpoint(object):
         # If an exception occurs then the success_response is None.
         # If no exception occurs then exception is None.
         success_response, exception = self._do_get_response(
-            request, operation_model)
+            request, operation_model, context
+        )
         kwargs_to_emit = {
             'response_dict': None,
             'parsed_response': None,
@@ -174,25 +248,30 @@ class Endpoint(object):
             http_response, parsed_response = success_response
             kwargs_to_emit['parsed_response'] = parsed_response
             kwargs_to_emit['response_dict'] = convert_to_response_dict(
-                http_response, operation_model)
+                http_response, operation_model
+            )
         service_id = operation_model.service_model.service_id.hyphenize()
         self._event_emitter.emit(
-            'response-received.%s.%s' % (
-                service_id, operation_model.name), **kwargs_to_emit)
+            f"response-received.{service_id}.{operation_model.name}",
+            **kwargs_to_emit,
+        )
         return success_response, exception
 
-    def _do_get_response(self, request, operation_model):
+    def _do_get_response(self, request, operation_model, context):
         try:
             logger.debug("Sending http request: %s", request)
-            history_recorder.record('HTTP_REQUEST', {
-                'method': request.method,
-                'headers': request.headers,
-                'streaming': operation_model.has_streaming_input,
-                'url': request.url,
-                'body': request.body
-            })
+            history_recorder.record(
+                'HTTP_REQUEST',
+                {
+                    'method': request.method,
+                    'headers': request.headers,
+                    'streaming': operation_model.has_streaming_input,
+                    'url': request.url,
+                    'body': request.body,
+                },
+            )
             service_id = operation_model.service_model.service_id.hyphenize()
-            event_name = 'before-send.%s.%s' % (service_id, operation_model.name)
+            event_name = f"before-send.{service_id}.{operation_model.name}"
             responses = self._event_emitter.emit(event_name, request=request)
             http_response = first_non_none_response(responses)
             if http_response is None:
@@ -200,21 +279,32 @@ class Endpoint(object):
         except HTTPClientError as e:
             return (None, e)
         except Exception as e:
-            logger.debug("Exception received when sending HTTP request.",
-                         exc_info=True)
+            logger.debug(
+                "Exception received when sending HTTP request.", exc_info=True
+            )
             return (None, e)
         # This returns the http_response and the parsed_data.
-        response_dict = convert_to_response_dict(http_response, operation_model)
+        response_dict = convert_to_response_dict(
+            http_response, operation_model
+        )
+        handle_checksum_body(
+            http_response,
+            response_dict,
+            context,
+            operation_model,
+        )
 
         http_response_record_dict = response_dict.copy()
-        http_response_record_dict['streaming'] = \
-            operation_model.has_streaming_output
+        http_response_record_dict[
+            'streaming'
+        ] = operation_model.has_streaming_output
         history_recorder.record('HTTP_RESPONSE', http_response_record_dict)
 
         protocol = operation_model.metadata['protocol']
         parser = self._response_parser_factory.create_parser(protocol)
         parsed_response = parser.parse(
-            response_dict, operation_model.output_shape)
+            response_dict, operation_model.output_shape
+        )
         # Do a second parsing pass to pick up on any modeled error fields
         # NOTE: Ideally, we would push this down into the parser classes but
         # they currently have no reference to the operation or service model
@@ -222,15 +312,20 @@ class Endpoint(object):
         # output shape but we can't change that now
         if http_response.status_code >= 300:
             self._add_modeled_error_fields(
-                response_dict, parsed_response,
-                operation_model, parser,
+                response_dict,
+                parsed_response,
+                operation_model,
+                parser,
             )
         history_recorder.record('PARSED_RESPONSE', parsed_response)
         return (http_response, parsed_response), None
 
     def _add_modeled_error_fields(
-            self, response_dict, parsed_response,
-            operation_model, parser,
+        self,
+        response_dict,
+        parsed_response,
+        operation_model,
+        parser,
     ):
         error_code = parsed_response.get("Error", {}).get("Code")
         if error_code is None:
@@ -243,24 +338,35 @@ class Endpoint(object):
         # TODO: avoid naming conflicts with ResponseMetadata and Error
         parsed_response.update(modeled_parse)
 
-    def _needs_retry(self, attempts, operation_model, request_dict,
-                     response=None, caught_exception=None):
+    def _needs_retry(
+        self,
+        attempts,
+        operation_model,
+        request_dict,
+        response=None,
+        caught_exception=None,
+    ):
         service_id = operation_model.service_model.service_id.hyphenize()
-        event_name = 'needs-retry.%s.%s' % (
-            service_id,
-            operation_model.name)
+        event_name = f"needs-retry.{service_id}.{operation_model.name}"
         responses = self._event_emitter.emit(
-            event_name, response=response, endpoint=self,
-            operation=operation_model, attempts=attempts,
-            caught_exception=caught_exception, request_dict=request_dict)
+            event_name,
+            response=response,
+            endpoint=self,
+            operation=operation_model,
+            attempts=attempts,
+            caught_exception=caught_exception,
+            request_dict=request_dict,
+        )
         handler_response = first_non_none_response(responses)
         if handler_response is None:
             return False
         else:
             # Request needs to be retried, and we need to sleep
             # for the specified number of times.
-            logger.debug("Response received to retry, sleeping for "
-                         "%s seconds", handler_response)
+            logger.debug(
+                "Response received to retry, sleeping for %s seconds",
+                handler_response,
+            )
             time.sleep(handler_response)
             return True
 
@@ -268,22 +374,30 @@ class Endpoint(object):
         return self.http_session.send(request)
 
 
-class EndpointCreator(object):
+class EndpointCreator:
     def __init__(self, event_emitter):
         self._event_emitter = event_emitter
 
-    def create_endpoint(self, service_model, region_name, endpoint_url,
-                        verify=None, response_parser_factory=None,
-                        timeout=DEFAULT_TIMEOUT,
-                        max_pool_connections=MAX_POOL_CONNECTIONS,
-                        http_session_cls=URLLib3Session,
-                        proxies=None,
-                        socket_options=None,
-                        client_cert=None,
-                        proxies_config=None):
-        if not is_valid_endpoint_url(endpoint_url):
-
+    def create_endpoint(
+        self,
+        service_model,
+        region_name,
+        endpoint_url,
+        verify=None,
+        response_parser_factory=None,
+        timeout=DEFAULT_TIMEOUT,
+        max_pool_connections=MAX_POOL_CONNECTIONS,
+        http_session_cls=URLLib3Session,
+        proxies=None,
+        socket_options=None,
+        client_cert=None,
+        proxies_config=None,
+    ):
+        if not is_valid_endpoint_url(
+            endpoint_url
+        ) and not is_valid_ipv6_endpoint_url(endpoint_url):
             raise ValueError("Invalid endpoint: %s" % endpoint_url)
+
         if proxies is None:
             proxies = self._get_proxies(endpoint_url)
         endpoint_prefix = service_model.endpoint_prefix
@@ -296,7 +410,7 @@ class EndpointCreator(object):
             max_pool_connections=max_pool_connections,
             socket_options=socket_options,
             client_cert=client_cert,
-            proxies_config=proxies_config
+            proxies_config=proxies_config,
         )
 
         return Endpoint(
@@ -304,7 +418,7 @@ class EndpointCreator(object):
             endpoint_prefix=endpoint_prefix,
             event_emitter=self._event_emitter,
             response_parser_factory=response_parser_factory,
-            http_session=http_session
+            http_session=http_session,
         )
 
     def _get_proxies(self, url):
