@@ -12,21 +12,30 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 
-import sys
 import logging
+from io import IOBase
 
-from botocore import ScalarTypes
-from botocore.hooks import first_non_none_response
-from botocore.compat import json, set_socket_timeout, XMLParseError
-from botocore.exceptions import IncompleteReadError, ReadTimeoutError
+from urllib3.exceptions import ProtocolError as URLLib3ProtocolError
 from urllib3.exceptions import ReadTimeoutError as URLLib3ReadTimeoutError
+
 from botocore import parsers
+from botocore.compat import set_socket_timeout
+from botocore.exceptions import (
+    IncompleteReadError,
+    ReadTimeoutError,
+    ResponseStreamingError,
+)
+
+# Keep these imported.  There's pre-existing code that uses them.
+from botocore import ScalarTypes  # noqa
+from botocore.compat import XMLParseError  # noqa
+from botocore.hooks import first_non_none_response  # noqa
 
 
 logger = logging.getLogger(__name__)
 
 
-class StreamingBody(object):
+class StreamingBody(IOBase):
     """Wrapper class for an http response body.
 
     This provides a few additional conveniences that do not exist
@@ -38,6 +47,7 @@ class StreamingBody(object):
           is raised.
 
     """
+
     _DEFAULT_CHUNK_SIZE = 1024
 
     def __init__(self, raw_stream, content_length):
@@ -45,6 +55,12 @@ class StreamingBody(object):
         self._content_length = content_length
         self._amount_read = 0
 
+    def __del__(self):
+        # Extending destructor in order to preserve the underlying raw_stream.
+        # The ability to add custom cleanup logic introduced in Python3.4+.
+        # https://www.python.org/dev/peps/pep-0442/
+        pass
+
     def set_socket_timeout(self, timeout):
         """Set the timeout seconds on the socket."""
         # The problem we're trying to solve is to prevent .read() calls from
@@ -58,16 +74,22 @@ class StreamingBody(object):
         # putting in a check here so in case this interface goes away, we'll
         # know.
         try:
-            # To further complicate things, the way to grab the
-            # underlying socket object from an HTTPResponse is different
-            # in py2 and py3.  So this code has been pushed to botocore.compat.
             set_socket_timeout(self._raw_stream, timeout)
         except AttributeError:
-            logger.error("Cannot access the socket object of "
-                         "a streaming response.  It's possible "
-                         "the interface has changed.", exc_info=True)
+            logger.error(
+                "Cannot access the socket object of "
+                "a streaming response.  It's possible "
+                "the interface has changed.",
+                exc_info=True,
+            )
             raise
 
+    def readable(self):
+        try:
+            return self._raw_stream.readable()
+        except AttributeError:
+            return False
+
     def read(self, amt=None):
         """Read at most amt bytes from the stream.
 
@@ -78,6 +100,8 @@ class StreamingBody(object):
         except URLLib3ReadTimeoutError as e:
             # TODO: the url will be None as urllib3 isn't setting it yet
             raise ReadTimeoutError(endpoint_url=e.url, error=e)
+        except URLLib3ProtocolError as e:
+            raise ResponseStreamingError(error=e)
         self._amount_read += len(chunk)
         if amt is None or (not chunk and amt > 0):
             # If the server sends empty contents or
@@ -86,14 +110,15 @@ class StreamingBody(object):
             self._verify_content_length()
         return chunk
 
+    def readlines(self):
+        return self._raw_stream.readlines()
+
     def __iter__(self):
-        """Return an iterator to yield 1k chunks from the raw stream.
-        """
+        """Return an iterator to yield 1k chunks from the raw stream."""
         return self.iter_chunks(self._DEFAULT_CHUNK_SIZE)
 
     def __next__(self):
-        """Return the next 1k chunk from the raw stream.
-        """
+        """Return the next 1k chunk from the raw stream."""
         current_chunk = self.read(self._DEFAULT_CHUNK_SIZE)
         if current_chunk:
             return current_chunk
@@ -101,7 +126,7 @@ class StreamingBody(object):
 
     next = __next__
 
-    def iter_lines(self, chunk_size=1024):
+    def iter_lines(self, chunk_size=_DEFAULT_CHUNK_SIZE, keepends=False):
         """Return an iterator to yield lines from the raw stream.
 
         This is achieved by reading chunk of bytes (of size chunk_size) at a
@@ -111,10 +136,10 @@ class StreamingBody(object):
         for chunk in self.iter_chunks(chunk_size):
             lines = (pending + chunk).splitlines(True)
             for line in lines[:-1]:
-                yield line.splitlines()[0]
+                yield line.splitlines(keepends)[0]
             pending = lines[-1]
         if pending:
-            yield pending.splitlines()[0]
+            yield pending.splitlines(keepends)[0]
 
     def iter_chunks(self, chunk_size=_DEFAULT_CHUNK_SIZE):
         """Return an iterator to yield chunks of chunk_size bytes from the raw
@@ -130,11 +155,16 @@ class StreamingBody(object):
         # See: https://github.com/kennethreitz/requests/issues/1855
         # Basically, our http library doesn't do this for us, so we have
         # to do this ourself.
-        if self._content_length is not None and \
-                self._amount_read != int(self._content_length):
+        if self._content_length is not None and self._amount_read != int(
+            self._content_length
+        ):
             raise IncompleteReadError(
                 actual_bytes=self._amount_read,
-                expected_bytes=int(self._content_length))
+                expected_bytes=int(self._content_length),
+            )
+
+    def tell(self):
+        return self._raw_stream.tell()
 
     def close(self):
         """Close the underlying http response stream."""
@@ -154,10 +184,12 @@ def get_response(operation_model, http_response):
         response_dict['body'] = http_response.content
     elif operation_model.has_streaming_output:
         response_dict['body'] = StreamingBody(
-            http_response.raw, response_dict['headers'].get('content-length'))
+            http_response.raw, response_dict['headers'].get('content-length')
+        )
     else:
         response_dict['body'] = http_response.content
 
     parser = parsers.create_parser(protocol)
-    return http_response, parser.parse(response_dict,
-                                       operation_model.output_shape)
+    return http_response, parser.parse(
+        response_dict, operation_model.output_shape
+    )
